module snf_adfmol
  !------------------------------------------------------------------------
  ! written by C. Herrmann, Uni Jena, April-June 2005
  !------------------------------------------------------------------------
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param
  use snf_adf, only : type_adf_ctrl

  private
  public ::  rwadfmol, read_adf_coord , read_adf_ctrl,check_adf_settings
  public ::  reorder_mol_adf

  !*********************************************************************

contains


      subroutine rwadfmol(natoms,ic,co,ir1,mxat,kcharg,rewri)
!
! read/write basic molecular input information from adf.in file
!
! written by C. Herrmann, Uni Jena 2005.
!
      use snf_molecule, only : find_nucl_charge
      implicit none
      integer mxat,mxsh,mxitem
      parameter(mxsh=50,mxitem=10)
      real(double) :: co(3,mxat)
      integer :: ic(mxat)
      integer :: ir1,lo,lr,lend,iat,j
      logical :: atomba,basis,intgrl,valbohr
      character(len=132) :: buf,bbuf,item(mxitem)
      character(len=2) :: symtxt
      character(len=4) :: rewri
      character(len=2) element(mxat)

      integer kcharg,nat,i,natoms,nsh,nsaos,nnsh,nont,nnsaos
      integer lam(mxat),nco(mxsh,mxat)
      character(len=1) :: charm
      character(len=72) ::  coform

       natoms=0
       i=1

        open(unit=ir1,file='adf.in.bak')
10      read (ir1,'(a)',ERR=900,END=900) buf
        call split(buf,item,mxitem)
! if Atom-block found, determine number of atoms
        if (item(1).eq.'Atoms'.or.item(1).eq.'ATOMS'.or.item(1).eq.'atoms') then
20        read(ir1,'(a)',ERR=900,END=900) buf
          call split(buf,item,mxitem)
          if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
            goto 30
          else
            co(1,i) = string2real(item(2))
            co(2,i) = string2real(item(3))
            co(3,i) = string2real(item(4))
            ic(i) =  find_nucl_charge(item(1)) 
            natoms = natoms+1
            i=i+1
            goto 20
          end if
! if there is a unit block specifying other than standard units (Bohr),
! set logical variable valbohr = .true.
        else if(item(2).eq.'Bohr'.or.item(2).eq.'BOHR'.or.item(2).eq.'bohr') then
          valbohr=.true.
        end if
       goto 10
30     continue
! if coordinates are given in Angstroem (ADF default), convert them to Bohr
! (Akira internal standard)
       if(.not.valbohr) then
         do i = 1,natoms
         do j=1,3
            co(j,i) =  co(j,i)*(1/bohr2ang)
         end do
         end do
         end if
900    continue
      close(ir1)
      end subroutine rwadfmol

!************************************************************************
      subroutine read_adf_coord(ctrl,molecule)
!
! read molecule information from the ADF input file adf.in
!
! written by C. Herrmann, University of Jena, 2005
!
      use snf_control, only : type_ctrl
      use snf_molecule, only : type_atom,find_element,find_mass,fiso_mass
      implicit none
      type(type_ctrl) :: ctrl
      type(type_atom),pointer :: molecule(:)
      integer li,lo,mxat,natoms,mcharge,nel,ncs,nos,i,calcno,nfreq
      parameter(mxat=1000)
      real(double) :: co(3,mxat)
      character(len=80) form,form2
      character(len=10) :: progtype,calctype
      integer :: ic(mxat)
      li=1
      !--------------------------------------------------------
      ! read geometry
      !--------------------------------------------------------
      call bfile('adf.in')
      call rwadfmol(natoms,ic,co,li,mxat,mcharge,'read')
      allocate(molecule(natoms))
      molecule(:)%changem=.false.
      do i=1,natoms
        molecule(i)%cartcoord(:)=co(:,i)
        molecule(i)%nucl_charge=ic(i)
        molecule(i)%element=find_element(ic(i))
        if(molecule(i)%element.eq.'D') then
          molecule(i)%mass=fiso_mass(1,2)
        else
          molecule(i)%mass=find_mass(molecule(i)%nucl_charge)
        end if
      end do

      end subroutine read_adf_coord

!************************************************************************

     subroutine check_adf_settings(adf_ctrl,molecule)
!
!  - read adf.in and check whether SCF-convergence criteria and
!    integration grid are sufficient. 
!  - write block requesting geometry optimization which is stopped
!    after one iteration (if not present yet)    
!
!  written by C. Herrmann, Uni Jena 06/2005
!  modified by J. Neugebauer, 10/2005:
!
!  - include RESPONSE block at this stage in case of Raman calculations
!  - check for SYMMETRY key and replace it by SYMMETRY NOSYM
!  - check for any STOPAFTER keys and replace them by STOPAFTER GGRADS
!  - check for settings that are important for well-converged SCF:
!    converge, occupations keeporbitals, occupations smearq
!
!--------------------------------------------------------------------
     use snf_molecule, only : type_atom,find_element,find_mass

     implicit none
!
     type(type_adf_ctrl), intent(in) :: adf_ctrl
     type(type_atom),pointer :: molecule(:)
     integer :: ir1,mxitem,mxbuf,i,ilast,ihalt,iatomstart,iatomend
     integer :: mxatoms,lscr,j,icount
     parameter(mxitem=10,mxbuf=300,mxatoms=500)
     character(len=132) :: buf(mxbuf),item(mxitem),scr
     character(len=2) :: el(mxatoms)
     real*8 :: rint1,rint2,rconv1,rconv2
     logical :: intchanged,intfound,scffound,convchanged,geofound,valbohr
     logical :: occfound
     integer :: lbuf(mxbuf)

     ir1=10
     intfound=.false.
     intchanged=.false.
     scffound=.false.
     geofound=.false.
     convchanged=.false.
     valbohr=.false.
     occfound=.false.
      open(unit=ir1,file='adf.in')
      i=1
10      read (ir1,'(a)',ERR=900,END=900) buf(i)
        call llen(buf(i),lbuf(i))
        call split(buf(i),item,mxitem)
        call upper(item(1))
        call upper(item(2))
        if(((item(1)(1:3).eq.'END').and.(item(2)(1:5).eq.'INPUT')).or. &
      & item(1)(1:8).eq.'ENDINPUT') then
            ihalt=i
        else if(item(1)(1:11).eq.'INTEGRATION') then
            goto 20
        else if(item(1)(1:3).eq.'SCF') then 
            goto 30
        else if(item(1)(1:8).eq.'GEOMETRY') then 
            goto 40
        else if(item(1).eq.'OCCUPATIONS') then
            write(buf(i),'(a)') 'OCCUPATIONS  KEEPORBITALS=1000000 SMEARQ=0.0d0'
            occfound=.true.
        else if(item(2).eq.'BOHR') then
          valbohr=.true.
        else if(item(1)(1:5).eq.'ATOMS') then
!----------------------------------------------------------------
! just skip ATOMS block. The new ATOMS block will be added below
!----------------------------------------------------------------
          i=i-1
          do
            read(ir1,'(a)',ERR=900,END=900) scr
            call llen(scr,lscr)
            call split(scr,item,mxitem)
            call upper(item(1))
            if(item(1)(1:3).eq.'END') exit
          end do
        else if(item(1)(1:8).eq.'RESPONSE') then
!----------------------------------------------------------------
! just skip response block: either, it is not needed in this
! calculation, or it will be added by snfdefine below
!----------------------------------------------------------------
          i=i-1
          do
            read(ir1,'(a)',ERR=900,END=900) scr
            call llen(scr,lscr)
            call split(scr,item,mxitem)
            call upper(item(1))
            if(item(1)(1:3).eq.'END') exit
          end do
        else if(item(1)(1:8).eq.'SYMMETRY') then
!----------------------------------------------------------------
! skip SYMMETRY key
!----------------------------------------------------------------
          i=i-1
        else if(item(1)(1:9).eq.'STOPAFTER') then
!----------------------------------------------------------------
! skip STOPAFTER keys
!----------------------------------------------------------------
          i=i-1
        end if
       goto 100
!----------------------------
20  rint1=string2real(item(2))
    rint2=string2real(item(3))
   if(rint1.lt.6.0d0) then
     rint1 = 6.0d0 
     intchanged=.true.
   else if(rint2.lt.6.0d0) then
     rint2 = 6.0d0
     intchanged=.true.
   end if
   write(buf(i),'(a12,2(f3.1,2x))') 'INTEGRATION ', rint1,rint2
   call llen(buf(i),lbuf(i))
    intfound=.true.
       goto 100
!----------------------------
30  i=i+1
     scffound=.true.
31  read(ir1,'(a)',ERR=900,END=900) buf(i)
    call llen(buf(i),lbuf(i))
    call split(buf(i),item,mxitem)
    call upper(item(1))
    if(item(1)(1:3).eq.'END') goto 100 
    if(item(1)(1:8).eq.'CONVERGE') then
       read(item(2),*) rconv1
       read(item(3),*) rconv2
      if(rconv1.gt.1.0d-6) then
        item(2)(1:6) = '1.0e-6'
        convchanged=.true.
      else if(rconv2.gt.1.0d-6) then
        item(3)(1:6) = '1.0e-6'
        convchanged=.true.
      end if
      write(buf(i),'(a10,2x,a6,2x,a6)') ' converge ', item(2)(1:6),item(3)(1:6) 
      call llen(buf(i),lbuf(i))
    end if
    i=i+1
    goto 31
!----------------------------
40    read(ir1,'(a)',ERR=900,END=900) scr
      call split(scr,item,mxitem)
      if(item(1)(1:3).eq.'end'.or.item(1)(1:3).eq.'END'.or.&
         &item(1)(1:3).eq.'End') then
         buf(i) =  'Geometry'
         call llen(buf(i),lbuf(i))
         buf(i+1) = '  GO'
         call llen(buf(i+1),lbuf(i+1))
         buf(i+2) = '  iterations 1'
         call llen(buf(i+2),lbuf(i+2))
         buf(i+3) = 'End'  
         call llen(buf(i+3),lbuf(i+3))
         i=i+3
         geofound=.true.
         goto 100
      end if
      goto 40
100    i=i+1
       goto 10
900  ilast=i-1
     rewind(ir1)
!-----------------------------------------
! skip empty lines at the end of the input
!-----------------------------------------
     icount=0
     do i=ihalt-1,1,-1
       if(len_trim(buf(i)).ne.0) exit
       icount=icount+1
     end do
     do i = 1,ihalt-1-icount
       call llen(buf(i),lbuf(i))
       write(ir1,'(a)') buf(i)(1:lbuf(i))
     end do
     write(ir1,'(a)')
     write(ir1,'(a)') 'SYMMETRY NOSYM'
     write(ir1,'(a)')
     write(ir1,'(a)') 'ATOMS'
111  format (1x,a2,4x,3(f20.14,2x))
     do i=1,size(molecule,1)
       if(valbohr) then
         write(ir1,111) molecule(i)%element,(molecule(i)%cartcoord(j),j=1,3)
       else
         write(ir1,111) molecule(i)%element,(molecule(i)%cartcoord(j)*bohr2ang,j=1,3)
       end if
     end do
     write(ir1,'(a)') 'END'
     write(ir1,'(a)')
     write(ir1,'(a)') 'STOPAFTER GGRADS'
     write(ir1,'(a)')
     if(.not.intfound) then
       write(ir1,'(a12,2(f3.1,2x))') 'INTEGRATION ', rint1,rint2
       intchanged=.true.
     end if
     if(adf_ctrl%ramanew.gt.0) then
       write(ir1,'(a)')
       write(ir1,'(a)') 'RESPONSE'
       write(ir1,'(a)') '  ALLCOMPONENTS'
       if(associated(adf_ctrl%dynfreq)) then
         write(ir1,'(a,i5)') '  NFREQ ',size(adf_ctrl%dynfreq,1)
         write(ir1,'(a,f24.16)') '  FRQBEG ',minval(adf_ctrl%dynfreq)
         write(ir1,'(a,f24.16)') '  FRQEND ',maxval(adf_ctrl%dynfreq)
         write(ir1,'(a)') '  HARTREE '
       end if
       write(ir1,'(a)') 'END' 
       write(ir1,'(a)')
     end if
     if(.not.scffound) then 
       write(ir1,'(a)')
       write(ir1,'(a)') 'SCF'
       write(ir1,'(a)') '  converge 1.0e-6 1.0e-6'
       write(ir1,'(a)') 'End' 
       write(ir1,'(a)')
       convchanged=.true.
     end if
     if(.not.geofound) then
       write(ir1,'(a)')
       write(ir1,'(a)') 'Geometry'
       write(ir1,'(a)') '  GO'
       write(ir1,'(a)') '  iterations 1'
       write(ir1,'(a)') 'End'
       write(ir1,'(a)')
     end if
     if(.not.occfound) then
       write(ir1,'(a)')
       write(ir1,'(a)') 'OCCUPATIONS KEEPORBITALS=1000000 SMEARQ=0.0d0'
       write(ir1,'(a)')
     end if
     do i = ihalt,ilast
       write(ir1,'(a)') buf(i)(1:lbuf(i))
     end do
     close(ir1)
     if(intchanged) then
       write(*,*) 
       write(*,*) '******************************************************'
       write(*,*) ' WARNING: The integration grid found in adf.in '
       write(*,*) ' is not sufficiently fine. It has been changed to' 
       write(*,'(a14,2(f3.1,2x))') '  INTEGRATION ', rint1,rint2 
       write(*,*) '******************************************************'
       write(*,*)
     end if
     if(convchanged) then
       write(*,*)
       write(*,*) '******************************************************'
       write(*,*) ' WARNING: The scf convergence criteria found in adf.in'
       write(*,*) ' are too loose. They have been changed to '
       write(*,'(a)') '  converge 1.0e-6 1.0e-6 '
       write(*,*) '******************************************************'
       write(*,*)
     end if
     if(intchanged.or.convchanged) then
       write(*,*) ' It is recommended to redo a geometry optimization '
       write(*,*) ' using the new input-settings '
       write(*,*)
       write(*,*) ' Please type <Enter> to confirm'
       write(*,*)
       read(*,'(a)') scr
     end if
     end subroutine check_adf_settings

!************************************************************************
      subroutine makeatomblocks(iatomstart,iatomend,el,mxatoms,buf,mxbuf)
!
! sort list of atom coordinates such that all atoms with the same
! nuclear charge are listed one after another.
!
! written by C. Herrmann, Uni Jena, June 2005
!
!
      implicit none
!
      integer :: iatomstart,iatomend,mxatoms,i,j,k,mxbuf
      integer :: jstart,istartnew,istartold,natoms
      character(len=2) :: el(mxatoms),elnew(mxatoms),selel
      character(len=132) :: buf(mxbuf),bufnew(mxbuf)
!
      natoms=iatomend-iatomstart-1
      jstart=iatomstart+1
      istartnew=iatomstart+1
      selel=el(jstart)
      do k = 1,natoms      
        istartold=istartnew
        call extractblock(bufnew,buf,selel,el,elnew,jstart,istartnew,istartold,natoms,mxatoms,mxbuf,iatomend)
        if(jstart.gt.iatomstart+natoms) exit
      end do
      do i = iatomstart+1,iatomend-1
        buf(i) = bufnew(i)
      end do
      end subroutine makeatomblocks
!************************************************************************
      subroutine extractblock(bufnew,buf,selel,el,elnew,jstart,istartnew,istartold,natoms,mxatoms,mxbuf,iatomend)
!
!  written by C. Herrmann, Uni Jena 06/2005
!
      implicit none
!
      integer :: i,j,k,mxatoms,mxbuf,iatomend
      integer :: jstart,istartnew,istartold,natoms
      logical newelfound
      character(len=2) :: el(mxatoms),elnew(mxatoms),selel,newselel
      character(len=132) :: buf(mxbuf),bufnew(mxbuf)
!
      newelfound=.false.
      j=jstart
      do i = istartold,iatomend
        if(el(i).eq.selel) then
          bufnew(j)=buf(i)
          j=j+1
        else if(el(i).ne.selel.and..not.newelfound) then
          istartnew=i 
          newelfound=.true.
          newselel=el(i)
        end if
      end do
      jstart=j
      selel=newselel
      end subroutine extractblock
!************************************************************************

      subroutine read_adf_ctrl(ctrl)
!
! read type of calculation for ADF calculation
!
! written by C. Herrmann, Uni Jena, 2005
!
      use snf_control, only : type_ctrl
      implicit none
      type(type_ctrl) :: ctrl

      end subroutine read_adf_ctrl

!************************************************************************

     subroutine reorder_mol_adf(molecule)
!
! sort molecules according to element symbols
!
!  written by J. Neugebauer, 10/2005
!
!--------------------------------------------------------------------

     use snf_molecule, only : type_atom,find_element,find_mass

     implicit none

     type(type_atom), pointer :: molecule(:)
     type(type_atom), pointer :: oldmolecule(:)
     integer :: norder(size(molecule,1)),natoms,icount
     character(len=2) :: selel,newselel
     integer :: i,j
     logical :: lnext

     natoms = size(molecule,1)
     allocate(oldmolecule(natoms))
     oldmolecule(:) = molecule(:)
     norder(1)=1
     do i=2,natoms
       norder(i) = 0
     end do

     selel=molecule(1)%element
     icount=1
     lnext=.true.
     do while(minval(norder).eq.0)
       if(.not.lnext) call serror('reorder_mol_adf',2,'error while reordering atoms')
       lnext=.false.
       do i=2,natoms
         if(norder(i).ne.0) cycle
         if(oldmolecule(i)%element==selel) then
           icount=icount+1
           norder(i)=icount
         else if(.not.lnext) then
           newselel=oldmolecule(i)%element
           lnext=.true.
         end if
       end do
       selel=newselel
     end do

     call serror('reorder_mol_adf',0,'Reordering atoms according to type')
     do i=1,natoms
!DBG       write(*,*) 'i, norder(i) : ',i,norder(i)
       molecule(norder(i)) = oldmolecule(i)
     end do
     deallocate(oldmolecule)

     end subroutine reorder_mol_adf

!************************************************************************

end module snf_adfmol
