module snf_gaussmol
  !------------------------------------------------------------------------
  ! written by C. Herrmann, Uni Jena, April-June 2005
  !------------------------------------------------------------------------
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param

  private
  public ::  rwgaussmol, read_gauss_coord , check_gauss_nosymm

  !*********************************************************************

contains


      subroutine rwgaussmol(natoms,ic,co,mxat)
!
! read/write basic molecular input information from gaussian.com file
!
! written by C. Herrmann, Uni Jena 2005.
!
      use snf_molecule, only : find_nucl_charge
      implicit none
      integer mxat,mxsh,mxitem,gunit,lrow,nempty,j
      parameter(mxsh=50,mxitem=10,gunit=17)
      real(double) :: co(3,mxat)
      integer :: ic(mxat)
      character(len=132) :: row,item(mxitem)
      character(len=2) el(mxat)
      integer kcharg,nat,i,natoms,nsh,nsaos,nnsh,nont,nnsaos
      nempty=0

! coordinates are given in Angstroem (Gaussian default), 
! so convert them to Bohr (SNF internal standard)
      open(gunit,file='gaussian.com',status='unknown')
1     read(gunit,'(a132)',end=2) row
      if(index(row(1:10),'#').ne.0) nempty=0
      call llen(row,lrow)
      if(row(1:10).eq.'          ') nempty = nempty +1
      if(nempty.eq.2) then
        i=1
        read(gunit,'(a132)',end=2) row
3       read(gunit,'(a132)',end=2) row
        if(row(1:10).ne.'          ') then
          read(row,*) el(i), co(1,i), co(2,i), co(3,i)
          ic(i)=find_nucl_charge(el(i))
          do j=1,3
             co(j,i) =  co(j,i)*(1/bohr2ang)
          end do
          i=i+1
          goto 3
        else
          goto 2
        end if
      end if
      goto 1
2     continue
      close(gunit)
      natoms = i-1

      end subroutine rwgaussmol

!************************************************************************
      subroutine read_gauss_coord(ctrl,molecule)
!
! read molecule information from the Gaussian input file gaussian.com
!
! written by C. Herrmann, University of Jena, 2005
!
      use snf_control, only : type_ctrl
      use snf_molecule, only : type_atom,find_element,find_mass,fiso_mass
      implicit none
      type(type_ctrl) :: ctrl
      type(type_atom),pointer :: molecule(:)
      integer li,lo,mxat,natoms,mcharge,nel,ncs,nos,i,calcno,nfreq
      parameter(mxat=1000)
      real(double) :: co(3,mxat)
      character(len=80) form,form2
      character(len=10) :: progtype,calctype
      integer :: ic(mxat)
      li=1
      !--------------------------------------------------------
      ! read geometry
      !--------------------------------------------------------
      call bfile('gaussian.com')
      call rwgaussmol(natoms,ic,co,mxat)
      allocate(molecule(natoms))
      molecule(:)%changem=.false.
      do i=1,natoms
        molecule(i)%cartcoord(:)=co(:,i)
        molecule(i)%nucl_charge=ic(i)
        molecule(i)%element=find_element(ic(i))
        if(molecule(i)%element.eq.'D') then
          molecule(i)%mass=fiso_mass(1,2)
        else
          molecule(i)%mass=find_mass(molecule(i)%nucl_charge)
        end if
      end do

      end subroutine read_gauss_coord

!************************************************************************

       subroutine check_gauss_nosymm
!
! check whether Keyword 'nosymm' is present in Gaussian input file - 
! if not, insert it.
!
! written by C. Herrmann, University of Jena, 2005
!
      implicit none
!
      integer :: gunit,mxrow,i,iend,mxitem,j,litem
      parameter(mxrow = 500,gunit=17,mxitem=20)
      integer :: lrow(mxrow)
      character(len=132) :: row(mxrow),item(mxitem)
!
      i=1
      open(gunit,file='gaussian.com',status='unknown')
1     read(gunit,'(a132)',end=2) row(i)
      call llen(row(i),lrow(i))
      call split(row(i),item,mxitem)
      if(i.eq.1) then
        row(i) = '#'
        lrow(i) = 1
        do j=2,mxitem
          call llen(item(j),litem)
          if(index(item(j),'symm').eq.0.and.index(item(j),'Symm').eq.0) then 
            row(i)=row(i)(1:lrow(i))//' '//item(j)(1:litem)
            call llen(row(i),lrow(i)) 
          end if 
        end do
        row(i) = row(i)(1:lrow(i))//' NoSymm'
        call llen(row(i),lrow(i))
      end if
      i=i+1
      goto 1
2     continue   
      iend=i-1 
      rewind(gunit)
      do i = 1,iend
        write(gunit,'(a)') row(i)(1:lrow(i))
      end do  
      close(gunit)

       end subroutine check_gauss_nosymm

end module snf_gaussmol
