
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_molp
  !------------------------------------------------------------------------
  ! written by S. Luber, ETH Zuerich, May 2006
  !------------------------------------------------------------------------
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param
  use snf_rbas,     only: read_tm_bas, type_basis
  
  private
  public :: type_molp_ctrl, init_molp_ctrl
  
  type type_molp_ctrl
    real(double),pointer :: dynfreq(:) ! angular frequencies for dyn. pol.
    character(len=132) :: coord    ! coord file in Molpro format
    character(len=132) :: control  ! control file for Molpro
    integer :: progno,  &          ! program type
         &     calcno,       &     ! calculation type
         &     ramanew             ! no. of polarizability sets
  end type type_molp_ctrl

  !*********************************************************************
 
contains
 
  !*********************************************************************
   
  subroutine init_molp_ctrl(ctrl)
    !----------------------------------------------------------------
    ! initialize default ctrl values for MOLPRO
    !
    ! written by : S. Luber, ETH Zuerich, 2006
    !---------------------------------------------------------------- 
    implicit none
    type(type_molp_ctrl), intent(out) :: ctrl
    nullify(ctrl%dynfreq)
    ctrl%progno=5
    ctrl%calcno=1
    ctrl%ramanew=0
    ctrl%coord='molpro.in'
    ctrl%control='molpro.in'
  end subroutine init_molp_ctrl
 
  !*********************************************************************
   
end module snf_molp
