module snf_molpmol
  !------------------------------------------------------------------------
  ! written by S. Luber, ETH Zuerich, 2006 
  !------------------------------------------------------------------------
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param

  private
  public ::  rwmolpmol, read_molp_coord

  !*********************************************************************

contains


    subroutine rwmolpmol(natoms,ic,co,mxat)
!
! read/write basic molecular input information from molpro.in file
! natoms: no. of atoms, ic: charge, co: coordinates, mxat: maximum no.
! of atoms
!
! written by S. Luber, ETH Zuerich 2006.
!
      use snf_molecule, only : find_nucl_charge
      implicit none
      integer mxat,mxsh,mxitem,gunit,lrow,z
      integer firstcomma,seccomma,thirdcomma,diffone,diffsec
      integer i,j,diffseco,diffthir,diffthird
           !i=1,2,...:No. of element
      parameter(mxsh=50,mxitem=10,gunit=17)
      double precision :: co(3,mxat)
!      real(double) :: co(3,mxat)
      integer :: ic(mxat)
      character(len=132) :: row,zeile,item(mxitem)
      character(len=3) :: el(mxat)
      integer kcharg,nat,natoms,nsh,nsaos,nnsh,nont,nnsaos

      open(gunit,file='molpro.in',status='unknown')
1     read(gunit,'(a132)',end=40) row
      if(index(row,'geometry').ne.0) then
        read(gunit,'(a132)',end=40) row
        row=trim(adjustl(row))
        if(row(1:10).ne.'           ') then
          read(row,*) natoms
          read(gunit,'(a132)',end=40) row
          i=1
2         read(gunit,'(a132)',end=40) zeile
          zeile=trim(adjustl(zeile))
          if(zeile(1:10).ne.'        ' .and. zeile(1:10).ne.'}') then
            firstcomma=index(zeile,',')
            diffone=firstcomma+1
            seccomma=index(zeile(diffone:132),',')+firstcomma
            diffseco=seccomma+1
            thirdcomma=index(zeile(diffseco:132),',')+seccomma
            diffsec=seccomma-1
            diffthir=thirdcomma-1
            diffthird=thirdcomma+1
            read(zeile(diffone:diffsec),*) co(1,i)
            read(zeile(diffseco:diffthir),*) co(2,i)
            read(zeile(diffthird:132),*) co(3,i)
            j=firstcomma-1
5           select case (zeile(j:j))
              case('0','1','2','3','4','5','6','7','8','9')
                j=j-1
                goto 5
              case default
                el(i)=zeile(1:j)
            end select
! coordinates in the inputfile are given in Angstroem
! convert them to Bohr         
            do z=1,3
               co(z,i)=co(z,i)*(1/bohr2ang)
            end do
            ic(i)=find_nucl_charge(el(i))
            i=i+1
            goto 2
          end if
        else
          write(*,*) 'No total number of atoms found'
        end if
      end if
      goto 1
40    continue
      close(gunit)

      end subroutine rwmolpmol

!***************************************************************************
      subroutine read_molp_coord(ctrl,molecule)
!
! read molecule information from the Molpro input file molpro.in
!
! written by S. Luber, ETH Zuerich, 2006
!
      use snf_control, only : type_ctrl
      use snf_molecule, only : type_atom,find_element,find_mass,fiso_mass
      implicit none
      type(type_ctrl) :: ctrl
      type(type_atom),pointer :: molecule(:)
      integer li,lo,mxat,natoms,mcharge,nel,ncs,nos,i,calcno,nfreq
      parameter(mxat=1000)
      real(double) :: co(3,mxat)
      character(len=80) form,form2
      character(len=10) :: progtype,calctype
      integer :: ic(mxat)
      li=1
     !--------------------------------------------------------
      ! read geometry
      !--------------------------------------------------------
      call bfile('molpro.in')
      call rwmolpmol(natoms,ic,co,mxat)
      allocate(molecule(natoms))
      molecule(:)%changem=.false.
      do i=1,natoms
        molecule(i)%cartcoord(:)=co(:,i)
        molecule(i)%nucl_charge=ic(i)
        molecule(i)%element=find_element(ic(i))
        if(molecule(i)%element.eq.'D') then
          molecule(i)%mass=fiso_mass(1,2)
        else
          molecule(i)%mass=find_mass(molecule(i)%nucl_charge)
        end if
      end do

      end subroutine read_molp_coord
end module snf_molpmol
