CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine adfrslave(grad,co,ilab,nmode,freq,intonly,nummodes)

c     ================================================================
c     purpose: perform an ADF single point calculation for a displaced
c              structure, collect the results, and send them to master
c 
c     history:
c       - written by C. Herrmann, Uni Jena, 05/2005
c       - modified by J. Neugebauer, VU Amsterdam, 2005
c       - cleaned and modified by J. Neugebauer, ETH Zurich, 2007
c     ================================================================

      include 'defs.h'
      include 'param.h'  

      integer ibuf,info,nn,iat,idum,ixyz,irc,idir,l
      integer idmy,j,ladf,iclean,icleanl,system
      real*8 escf
      real*8 nmode(maxmode,maxatoms*3),freq(maxmode)

      integer xprog
      integer*4 jstepa,nit
      real*8 cpu,wall,gcpu,gwall
      real*8 grad(nvar),dipol(3),co(3,natoms),polar(8,mxomeg)
      real*8 gtenlao(9,mxomeg),gtenao(9,mxomeg),aten(27,mxomeg)
      integer ilab(natoms)

      logical found
      integer intonly,nummodes
      character*5 nic
      character*255 adfname,b1
      character*255 forceout,adfdipin,adfdipout,adfpolin,adfpolout
      integer lfout,ladfdi,ladfdo,ladfpoli,ladfpolo

c     -------------------------------
c     prepare input by calling rslave
c     -------------------------------
      call rslave(co,ilab,jstepa,nmode,freq,intonly,nummodes,nic)

c     ------------------------------------------------------------------
c     remove TAPE21 and t21* files from previous ADF runs
c     (otherwise, ADF will not run properly)
c     ------------------------------------------------------------------
      info=system('rm '//tempdir(1:lt)//'/'//'TAPE21 t21*')

c     -------------------------
c     prepare in-/output file names
c     -------------------------
      call augnum(adfname,'adf.out.',jstepa)
      call llen(adfname,ladf)
      call augnum(adfdipin,'adfdip.in.',jstepa)
      call llen(adfdipin,ladfdi)
      call augnum(adfdipout,'adfdip.out.',jstepa)
      call llen(adfdipout,ladfdo)

c     ------------------------------------
c     run ADF
c     ------------------------------------
      info=system('chmod u+x '//tempdir(1:lt)//'/adfslave.in')
      info=xprog(nic(1:5)//tempdir(1:lt)//'/adfslave.in',
     *     tempdir(1:lt)//'/'//adfname(1:ladf))

c     ------------------------------------------------------------------
c     extract gradient information from TAPE21
c     ------------------------------------------------------------------
      call augnum(forceout,'grad.out.',jstepa)
      call llen(forceout,lfout)
      info=xprog(nic(1:5)//'$ADFBIN/dmpkf '//tempdir(1:lt)//
     * '/TAPE21 Geometry GeoOpt',
     *   tempdir(1:lt)//'/'//forceout(1:lfout))

c     ------------------------------------
c     read gradient 
c     ------------------------------------
      CALL gadfgrad(natoms,grad,tempdir(1:lt)//'/'
     *                //forceout(1:lfout))

c     ----------------------------------------------------------
c     get dipole moments
c     -----------------------------------------------------------
      call clean(adfdipout)
      call augnum(adfdipout,'dip.out.',jstepa)
      call llen(adfdipout,lfout)
      info=xprog(nic(1:5)//'$ADFBIN/dmpkf '//tempdir(1:lt)//
     *'/TAPE21 Properties',
     *  tempdir(1:lt)//'/'//adfdipout(1:lfout))
      call gadfdip2(dipol,tempdir(1:lt)//'/'//adfdipout(1:lfout))

c     -----------------------------------------------------------
c     get polarizabilities
c     -----------------------------------------------------------
      if(ipol.ne.0) then
        call gadfpol(polar,tempdir(1:lt)//'/'//adfname(1:ladf),iosl)
      end if
      nit=0
      cpu=0.d0
      gcpu=0.d0
      wall=0.d0
      gwall=0.d0
      call mnotify(5,mytid,6,'times read',10)

c     --------------------------------------------------
c     read and save total bonding energy
c     --------------------------------------------------
      call gadfenergy(tempdir(1:lt)//'/'//adfname(1:ladf),escf)
      call mnotify(5,mytid,5,'results read',12)

c     ------------------
c     send final results
c     ------------------
      call sendres(nit,cpu,wall,gcpu,gwall,escf,dipol,grad,
     *   polar,gtenlao,gtenao,aten,jstepa,nummodes,intonly)

      return
      end                        
