CC****************************************************************************** C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************

      subroutine basic_input

c     ==========================================================
c     purpose: read basic input files for snfdc and electronic
c              structure programs
c
c     history: 
c      - template subroutine tmini written by B.A. Hess
c      - re-written by J. Neugebauer, Uni Erlangen, 2003
c      - modified by C. Herrmann, Uni Bonn, 02/2005 (intonly tool)
c      - modified by C. Herrmann, Uni Jena, 04/2005 (ADF interface)
c      - modified by C. Herrmann, Uni Jena, 05/2005 (Gaussian interface)
c      - modified by S. Luber, ETH Zuerich, 06/2006 (Molpro interface)
c     ==========================================================

      include 'defs.h'
      include 'param.h'
      include 'objects.h'

      integer i,nn,idalmw,iclean,icleanl,nfex,ir1,mcharge,lo,j,lsco
      integer mxat
      character*255 b1
      character*80 dalmw
      logical snffast
      real*8 charge,freq(maxmode),nmode(maxmode,maxatoms*3)
      character*10 calctype,prtype
      integer intonly,nummodes
      external nfex
      ir1=3
      lo=2
      
c     ---------------------
c     read snf_control file
c     ---------------------
      call slen(mydir(1:lm)//'/snf_control',lsco)
      call rscontrol(mydir(1:lm)//'/snf_control',progno,calcno,ipol,
     *  omeg,snffast,iclean,icleanl,naus,mxomeg,cstep,lastqmatom,
     *  irescp,restadf,para,lsco)


c     ----------------------------
c     Turbomole: read control file
c     ----------------------------
      if(progno.eq.1) then
        call rcontrol(mydir(1:lm)//'/control',rpast,smult,isuhf,
     *    natoms,idbg)

c     ----------------------------
c     Dalton: read control file
c     ----------------------------
      else if(progno.eq.2) then
        open(ir1,file=mydir(1:lm)//'/molecule.mol')
        call rbdalmol(natoms,ir1,mcharge)
        close(ir1)

c     ----------------------------
c     Gaussian: read control file
c     ----------------------------
      else if(progno.eq.3) then
         call rgauss(mydir(1:lm)//'/gaussian.com',natoms)

c     ----------------------------
c     ADF: read control file
c     ----------------------------
      else if(progno.eq.4) then
         call radf(mydir(1:lm)//'/adf.in',natoms)

c     ------------------------------
c     Molpro: read control file
c     ------------------------------
      else if(progno.eq.5) then
         call rmolp(mydir(1:lm)//'/molpro.in',natoms)

      else 
        write(jtape,*) 'program mode currently not available'
        call errex('program mode not available')
      end if

c     -------------------------------------
c     determine code for file copy checking
c     -------------------------------------
      okok=nfex(progno,calcno,isuhf,irescp)
      write(idbg,*) 'okok : ',okok
      call flush(idbg)
      if(okok.le.0) call errex('basic_input | invalid okok value')

      write (jtape,*) 
      write (jtape,'(A)') 
     *  ' ===================================================== '
      write (jtape,'(A)') 
     *  ' == SNFDC INPUT PARAMETERS FROM CONTROL AND RESTART == '
      write (jtape,'(A)') 
     *  ' ===================================================== '
      write (jtape,*) 

c     ------------------------------------------------------
c     NOTE: We should actually set intonly in rscontrol, and
c           call rscontrol2 only when needed !!!
c
c     intensities only mode: read frequencies 
c     and normal modes from snf_control
c     ------------------------------------------------------
      call rscontrol2(nmode,nummodes,intonly,freq,
     *     mydir(1:lm)//'/snf_control',idbg)

c     ----------------------------------------------------------
c     determine type of program to be used for single points and 
c     type of calculation
c     ----------------------------------------------------------
      call prog_type(progno,prtype)
      call calc_type(progno,calcno,calctype)
      write (jtape,'('' number of atoms             '',i5)') natoms
      write (jtape,'('' degrees of freedom (d.o.f.) '',i5)') natoms*3
      write (jtape,'('' No. of steps per d.o.f.     '',i5)') naus
      write (*,'('' step sizes (bohr)'',f16.5)') cstep
      write (jtape,'('' program for single points   '',a10)') prtype
      write (jtape,'('' type of calculations        '',a10)') calctype
      write (jtape,'('' no. of freq-dep tensor sets '',i5)') ipol
      if(intonly .ne. 0) then
         write(jtape,'(a)') ' intensities-only calculation requested'
      end if
      if(snffast) then
         write(*,*) 
     *     'Omitting command <nice> for single point calculations'
      end if
      if(iclean.eq.1) 
     *write(*,'('' remove directories for single point calculations'')')
      if(icleanl.eq.1) 
     *  write(*,'('' remove log files from slave processes'')')
      if (icrap.eq.0 .and. (cstep.lt.0.01.or.cstep.gt.0.05)) then
        write (jtape,*)
     *   'The present parameter setting of SNFDC enforces'
        write (jtape,*) 'a cstep value 0.01 <= cstep <= 0.05.'
        write (jtape,*) '$cstep has been given as ',cstep
        write (jtape,*)
     *   'set CRAP_OK in the SNFDC input, if this is OK'
        write (jtape,*)
     *   'note that this may give much less accurate results'
        write (jtape,*) 'for frequency and intensities.'
        call errex('cstep outside recommended range')
      end if
      
c     ----------------
c     read coordinates
c     ----------------

      call memmgr(memget,'ilab    ',is,plab,natoms)
      call memmgr(memgdw,'co      ',s8,pco,natoms*3)

      if(progno.eq.1) then
        call rcoord(mydir(1:lm)//'/coord',nn,s8(pco),is(plab))
      else if(progno.eq.2) then
        open(ir1,file=mydir(1:lm)//'/molecule.mol')
        nn=natoms
        mxat=natoms
        call rwdalmol(nn,is(plab),s8(pco),ir1,mxat,mcharge,'read',lo)
        close(ir1)
      else if(progno.eq.3) then
        call rgausscoord(mydir(1:lm)//'/gaussian.com',natoms,s8(pco),
     *  is(plab))
        nn=natoms
      else if(progno.eq.4) then
        call radfcoord(mydir(1:lm)//'/adf.in',natoms,s8(pco),
     *  is(plab),iosl)
        nn=natoms
      else if(progno.eq.5) then
        call rmolpcoord(mydir(1:lm)//'/molpro.in',natoms,s8(pco),
     *  is(plab),iosl)
        nn=natoms
      end if

      if (nn.NE.natoms) THEN
        write (*,*) '###### natoms from control and coord differ!'
C~      call errex('basic_input: incorrect natoms')
C//SKIPC~1
        call shutdown
      end if

      write (jtape,*) ' '
      write (jtape,*)
     *  '        ORIENTATION OF MOLECULE IN FORCE CALCULATION'
      write (jtape,*) ' '
      write (jtape,*)
     *  '   NO.       ATOM         X         Y         Z'
      write (jtape,*) ' '
      call wcoo(natoms,is(plab),s8(pco))

      nvar =natoms*3
      if(intonly.eq.1) then
        nstept=nummodes*naus
      else
        nstept=nvar*naus
      end if
      
      write (jtape,*)
      write (jtape,'(A)') 
     *  ' ===================================================== '
      write (jtape,*)
      return
      end

      subroutine wcoo(natoms,ilab,co)
      include 'defs.h'
      include 'fconstants.h'
      integer natoms,i
      integer ilab(natoms)
      real*8 co(3,natoms)

      do i=1,natoms
        write (*,'(i6,i10,4x,3f10.4)') i,ilab(i),
     *   co(1,i)*bohr2ang,co(2,i)*bohr2ang,co(3,i)*bohr2ang
      end do

      return
      end
