      subroutine copyfiles(info,mynode,pnode)

c     =========================================================
c     copy files needed by electronic-structure programs on 
c     slaves (for displaced structures)
c
c     written by J. Neugebauer, ETHZ, 01/2007
c     (extracted from islave)
c     =========================================================
      include 'defs.h'
      include 'param.h'
      integer info,ir1,mcharge,l,xeq,irc
      character*10 calctype
      character*(*) mynode,pnode
      character*255 b1
 
      call calc_type(progno,calcno,calctype)

c     ----------------------------------------------------
c     program specific files
c     ----------------------------------------------------

c     -------------------------------------
c     Turbomole: copy and read control file
c     -------------------------------------
      if(progno.eq.1) then
        call getfile('control',mynode,pnode,tid0,info)
        call rcontrol(tempdir(1:lt)//'/control',rpast,smult,isuhf,
     *   natoms,idbg)
c     ---------------------------------------
c     Dalton: copy and read molecule.mol file
c     ---------------------------------------
      else if(progno.eq.2) then
        call getfile('molecule.mol',mynode,pnode,tid0,info)
        ir1=48
        open(ir1,file=tempdir(1:lt)//'/molecule.mol')
        call rbdalmol(natoms,ir1,mcharge)
        close(ir1)
        call copy_ori('molecule.mol')

c     ---------------------------------------
c     Gaussian: copy and read gaussian.com file
c     ---------------------------------------
      else if(progno.eq.3) then
        call getfile('gaussian.com',mynode,pnode,tid0,info)
        call rgauss(tempdir(1:lt)//'/gaussian.com',natoms)
        call copy_ori('gaussian.com')

c     ---------------------------------------
c     ADF: copy and read adf.in file
c     ---------------------------------------
      else if(progno.eq.4) then
        call getfile('adf.in',mynode,pnode,tid0,info)
        call radf(tempdir(1:lt)//'/adf.in',natoms)
        call copy_ori('adf.in')

c     -----------------------------------------
c     Molpro: copy and read molpro.in file
c     -----------------------------------------
      else if(progno.eq.5) then
        call getfile('molpro.in',mynode,pnode,tid0,info)
        call rmolp(tempdir(1:lt)//'/molpro.in',natoms)
        call copy_ori('molpro.in')

      end if

c     ----------------------------------------------------
c     calculation-type specific files
c     ----------------------------------------------------

c     -----------------------
c     Turbomole: coord, basis
c     -----------------------
      if(progno.eq.1) then
        call getfile('coord',mynode,pnode,tid0,info)
        call copy_ori('coord')

        call getfile('basis',mynode,pnode,tid0,info)

c       -------------------------------------
c       Turbomole: rks/rhf : mos
c                  uks/uhf : alpha,beta
c       -------------------------------------
        if (isuhf.EQ.0) THEN
          call getfile('mos',mynode,pnode,tid0,info)
          call copy_ori('mos')

        else
          call getfile('alpha',mynode,pnode,tid0,info)
          call getfile('beta',mynode,pnode,tid0,info)
          call copy_ori('alpha')
          call copy_ori('beta')

        end if

c       -----------------------------------------
c       Turbomole: ridft/rimp2 ? -> copy auxbasis
c       -----------------------------------------
        if(index(calctype,'RI').ne.0) ridft=1
        if(ridft.gt.0) then
          call getfile('auxbasis',mynode,pnode,tid0,info)
        end if

c     ----------------------------------
c     Dalton: molecule.dal, molecule.mol
c     ----------------------------------
      else if(progno.eq.2) then
        call getfile('molecule.dal',mynode,pnode,tid0,info)

c       -----------------------------------------
c       Dalton: MP2/SOPPA: 2 .dal files necessary
c       -----------------------------------------
        if(index(calctype,'SOPPA').ne.0) then
          call getfile('molecule.dal.B',mynode,pnode,tid0,info)
        end if

c     ----------------------------------
c     Gaussian: no further files needed
c     ----------------------------------
      else if(progno.eq.3) then
        continue

c     ----------------------------------
c     ADF: no further files needed
c     ----------------------------------
      else if(progno.eq.4) then
        continue

c     ----------------------------------
c     MOLPRO: no further files needed
c     ----------------------------------
      else if(progno.eq.5) then
        continue

      else
        call errex('program type not implemented')

      end if
      call flush(iosl)

      end

**************************************************************************

      subroutine copy_ori(fname)

c     ===================================================
c     purpose: move file <tempdir/fname> to 
c              <tempdir/fname.ori>
c
c     written by J. Neugebauer, ETHZ, 01/2007
c     ===================================================

      include 'defs.h'
      include 'param.h'

      integer irc,xeq,lf,l
      character*255 b1
      character*(*) fname

      call slen(fname,lf)
      call clean(b1)
      b1=tempdir(1:lt)//'/'//fname(1:lf)
      call llen(b1,l)
      irc=xeq('cp '//b1(1:l)//' '//b1(1:l)//'.ori')

      end
