CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine daltonrslave(grad,co,ilab,nmode,freq,intonly,nummodes)

c     ==================================================================
c     purpose: perform a DALTON single point calculation for a displaced
c              structure, collect the results, and send them to master
c 
c     history:
c       - written by J. Neugebauer, Uni Erlangen, 2003
c       - extended by M. Reiher, Uni Erlangen, June 2003
c       - modified by M. Reiher, Uni Bonn, April 2004
c       - modified by C. Herrmann, 02/2005
c       - cleaned and modified by J. Neugebauer, ETH Zurich, 2007
c     ==================================================================

      include 'defs.h'
      include 'param.h'  

      integer ibuf,info,nn,iat,idum,ixyz,irc,idir,l
      integer idmy,j,diraus,ldal,iclean,icleanl
      character*255 dalname
      real*8 escf
      real*8 nmode(maxmode,maxatoms*3),freq(maxmode)

      integer xprogb,xprog,ldmol
      integer*4 itst,jstepa
      character*255 b1
      character*5 nic
      character*80 dalmw
      character*24 dmname
      real*8 cpu,wall,gcpu,gwall
      integer*4 nit
      integer xeq,tmpxeq,idalmw,ldalmw
      real*8 grad(nvar),dipol(3),co(3,natoms),polar(8,mxomeg)
      real*8 gtenlao(9,mxomeg),gtenao(9,mxomeg),aten(27,mxomeg)
      integer ilab(natoms)
      integer intonly,nummodes
      logical found,vroaflag


      vroaflag=.false.
      if ((ipol-1000).ge.0) vroaflag=.true.

      idalmw=0
      call clean(dalmw)

*-------------------
* call rslave module
*-------------------
      call rslave(co,ilab,jstepa,nmode,freq,intonly,nummodes,nic)

*--------------------------
* prepare output file names
*--------------------------
      call augnum(dalname,'dal.out.',jstepa)
      call llen(dalname,ldal)
      call llen(dalmw,ldalmw)
      call augnum(dmname,'molecule',jstepa)
      call llen(dmname,ldmol)
      info=xeq('mv '//tempdir(1:lt)//'/molecule.mol '
     &     //tempdir(1:lt)//'/'//dmname(1:ldmol)//'.mol')
      info=xeq('cp '//tempdir(1:lt)//'/molecule.dal '
     &     //tempdir(1:lt)//'/'//dmname(1:ldmol)//'.dal')
      if(para) then
        info=xeq('cp '//tempdir(1:lt)//'/'//dmname(1:ldmol)//'.mol '
     &     //tempdir(1:lt)//'/MOLECULE.INP')
        info=xeq('cp '//tempdir(1:lt)//'/'//dmname(1:ldmol)//'.dal '
     &     //tempdir(1:lt)//'/DALTON.INP')
      end if

      if(.not.para) then
*--------------------------------------------------------------
* set maximum scratch memory if dalmw is specified in "control"
*--------------------------------------------------------------
        if(idalmw.le.800000) then
          if(idalmw.ne.0) then
            write(idbg,*) 
     &   'rslave: using dalton with default scratch memory size'
            write(idbg,*) 'idalmw = ',idalmw
           end if
*-----------
* run DALTON
*-----------
           info=xprog(nic(1:5)//progdir(1:lb)//'dalton '
     &   //dmname(1:ldmol)//' '//dmname(1:ldmol),dalname(1:ldal))
        else 
           info=xprog(nic(1:5)//progdir(1:lb)//'dalton -m '
     &   //dalmw(1:ldalmw)//' '//dmname(1:ldmol)//' '//dmname(1:ldmol),
     &   dalname(1:ldal))
        end if
      else
        info=xprog(nic(1:5)//progdir(1:lb)//'mpiexec -conf '
     & //tempdir(1:lt)//'/dalpar.config ',dalname(1:ldal)) 
      end if
*-------------------
* get dipole moments
*-------------------
      if((calcno.ge.2.and.calcno.le.5).or.calcno.eq.8) then
        call gdaldip(dipol,tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
      else if(calcno.eq.1.or.calcno.eq.6.or.calcno.eq.7.or.
     &      calcno.eq.9) then
        call gdaldipscf(dipol,
     &   tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
      end if
      nit=0
      cpu=0
      gcpu=0
      wall=0
      gwall=0
      call mnotify(5,mytid,6,'times read',10)

*---------------------------------------------------------------------------------
* get VROA tensors as well as gradients if VROA is switched on; then, jump to the end 666 
*---------------------------------------------------------------------------------
      if (vroaflag) then
        if(calcno.eq.1.or.calcno.eq.6.or.calcno.eq.9) then
          call gdalgradscf(natoms,escf,grad,
     *     tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
           call gdalvroascf(polar,gtenlao,gtenao,aten,
     *     tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
        else
          call errex('calcno value not possible for VROA!')
        end if
        goto 666
      end if
*** if VROA is switched off proceed as usual:
*-------------------------------------
* read gradient (and polarizabilities)
*-------------------------------------
      if(calcno.ge.2.and.calcno.le.5) then
        call gdalgrad(natoms,escf,grad,
     *   tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
        if(ipol.gt.0)
     *   call gdalpol(polar,
     *   tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
      else if(calcno.eq.1.or.calcno.eq.6.or.calcno.eq.9) then
        call gdalgradscf(natoms,escf,grad,
     *   tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
        if(ipol.gt.0)
     *   call gdalpolscf(polar,
     *   tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
      else if(calcno.eq.7) then
        call gdalgradscf(natoms,escf,grad,
     *   tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
        if (ipol.GT.0)
     *   call gdalpoltdhf(polar,
     *   tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
      else if(calcno.eq.8) then
        call gdalgrad(natoms,escf,grad,
     *   tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out')
        call augnum(dmname,tempdir(1:lt)//'/molecule.out.A.',jstepa)
        call llen(dmname,ldmol)
        irc=xeq('mv '//tempdir(1:lt)//'/molecule.out '//dmname(1:ldmol))

*------------------------------------------------
* special treatment in case of SOPPA calculations
*------------------------------------------------
        if (ipol.gt.0) then
          info=xeq('mv '//tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out '
     &     //tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out.A')
          info=xeq('cp '//tempdir(1:lt)//'/molecule.dal.B '
     &     //tempdir(1:lt)//'/'//dmname(1:ldmol)//'.dal ')
          if(idalmw.le.800000) then
            if(idalmw.ne.0) then
             write(idbg,*) 
     &    'rslave: using dalton with default scratch memory size'
             write(idbg,*) 'idalmw = ',idalmw
            end if
            call flush(idbg)
            info=xprog(nic(1:5)//progdir(1:lb)//'dalton '
     &       //dmname(1:ldmol)//' '//dmname(1:ldmol),dalname(1:ldal))
          else 
            info=xprog(nic(1:5)//progdir(1:lb)//'dalton -m '
     &       //dalmw(1:ldalmw)//' '//dmname(1:ldmol)//' '
     &       //dmname(1:ldmol),dalname(1:ldal))
          end if
          info=xeq('mv '//tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out ' 
     &     //tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out.B')
          call gdalpoltdhf(polar,
     *      tempdir(1:lt)//'/'//dmname(1:ldmol)//'.out.B')
        end if
      end if

666   continue

      call mnotify(5,mytid,5,'results read',12)

c     ------------------
c     send final results
c     ------------------
      call sendres(nit,cpu,wall,gcpu,gwall,escf,dipol,grad,
     *   polar,gtenlao,gtenao,aten,jstepa,nummodes,intonly)

      return
      end                        
