CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine g98rslave(grad,co,ilab,nmode,freq,intonly,nummodes)

c     ====================================================================
c     purpose: perform a GAUSSIAN single point calculation for a displaced
c              structure, collect the results, and send them to master
c 
c     history:
c       - written by C. Herrmann, Uni Jena, 05/2005
c       - modified by C. Herrmann, 07/2006
c       - cleaned and modified by J. Neugebauer, ETH Zurich, 2007
c     ====================================================================

      include 'defs.h'
      include 'param.h'  

      integer info,iat,l
      integer j,diraus,iclean,icleanl,system
      real*8 escf
      real*8 nmode(maxmode,maxatoms*3),freq(maxmode)

      integer xprogb,xprog
      integer*4 jstepa
      character*255 b1
      character*5 nic
      character*255 forceout,forcein,polout
      real*8 cpu,wall,gcpu,gwall
      integer*4 nit
      integer xeq,tmpxeq
      real*8 grad(nvar),dipol(3),co(3,natoms),polar(8,mxomeg)
      real*8 gtenlao(9,mxomeg),gtenao(9,mxomeg),aten(27,mxomeg)
      integer ilab(natoms),lfout,lfin,lpolo
      logical found
      integer intonly,nummodes,i

*-------------------
* call rslave module
*-------------------
      call rslave(co,ilab,jstepa,nmode,freq,intonly,nummodes,nic)

*--------------------------
* prepare in-/output file names
*--------------------------
      CALL augnum(forcein,'force.com.',jstepa)
      CALL llen(forcein,lfin)
      info=system('mv '//tempdir(1:lt)//'/force.com '//tempdir(1:lt)//
     *  '/'//forcein(1:lfin))
      CALL augnum(forceout,'force.log.',jstepa)
      CALL llen(forceout,lfout)
      if(calcno.eq.1) then
        write(iosl,*) '** rslavem30: running gaussian98'
        call flush(iosl)
        info=xprog(nic(1:5)//progdir(1:lb)//'g98 < '//tempdir(1:lt)//
     *  '/'//forcein(1:lfin),tempdir(1:lt)//'/'//forceout(1:lfout))
      else if(calcno.eq.2) then
         write(iosl,*) '** rslavem30: running gaussian03'
        call flush(iosl)
        info=xprog(nic(1:5)//progdir(1:lb)//'g03 < ' //tempdir(1:lt)//
     *  '/'//forcein(1:lfin),tempdir(1:lt)//'/'//forceout(1:lfout))
      end if

      CALL ggaussgrad(natoms,escf,dipol,grad,tempdir(1:lt)//'/'
     *                //forceout(1:lfout))
      write(iosl,*) '** rslavem30: gradient read'
      CALL flush(iosl)

      if(info.ne.0) call errex('rslavem30: system command failed')
      CALL flush(iosl)

*------------------------------------------------------------
* get polarizabilities:
* 1) prepare input file for single-point calculation
* 2) perform single-point calculation
* 4) extract components of polarizability from output
*------------------------------------------------------------
      if(ipol.ne.0) then
        call augnum(polout,'polar.log.',jstepa)
        call llen(polout,lpolo)
        call wgausspol(tempdir(1:lt),forcein(1:lfin),'polar.com',iosl)
        if(calcno.eq.1) then
          info=xprog(nic(1:5)//progdir(1:lb)//'g98 ' //tempdir(1:lt)//
     *     '/polar.com','polar.log')
        else if(calcno.eq.2) then
          info=xprog(nic(1:5)//progdir(1:lb)//'g03 ' //tempdir(1:lt)//
     *     '/polar.com','polar.log')
        end if
        call ggausspol(polar,tempdir(1:lt)//'/polar.log',iosl)
        info=system('mv '//tempdir(1:lt)//'/polar.log '
     *                //tempdir(1:lt)//'/'//polout(1:lpolo))
        write(iosl,*) 'rslavem30: mv '//tempdir(1:lt)//'/polar.log '
     *                //tempdir(1:lt)//'/'//polout(1:lpolo) 
        CALL flush(iosl)
        if(info.ne.0) call errex('rslavem30: system command failed')
        CALL flush(iosl)
      end if

      cpu=0.0d0
      wall=0.0d0
      gcpu=0.0d0
      gwall=0.0d0

c     ------------------
c     send final results
c     ------------------
      call sendres(nit,cpu,wall,gcpu,gwall,escf,dipol,grad,
     *   polar,gtenlao,gtenao,aten,jstepa,nummodes,intonly)

      return
      end                        
