CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************

      subroutine getfile(name,mynode,pnode,tid,info)

c     ====================================================
c     purpose: copy files from master to slave using
c              either scp/cp  (-> getfile_scp)
c              or     pvm/mpi (-> getfile_pvmpi)
c
c              this routine thus provides a unique
c              interface to copy files with whatever
c              configure option
c
c     written by J. Neugebauer, ETHZ, 01/2007
c
c     NOTE: pvm/mpi copies must of course have an sfile
c           counterpart !
c     ====================================================

      include 'defs.h'
      character*(*) name,mynode,pnode
      integer info,tid

C~    call getfile_scp(name,mynode,pnode,info)
C//SKIPC~
C&    call getfile_scp(name,mynode,pnode,info)
C//SKIPC&1
      call getfile_pvmpi(name,tid,info)
C//END

      return
      end 

c****************************************************************************

      subroutine getfile_scp(name,mynode,pnode,info)

c     =====================================================
c     purpose: copy files from master to slave using scp/cp
c
c     variables used from common blocks:
c     - tempdir,mydir,lt,lm,mytid            
c
c     - replaced mydir by workdir (J. Neugebauer, 09/2005)
c     - changed again by C. Herrmann
c     =====================================================

      implicit none
      include 'param.h'
      character*(*) name,mynode,pnode
      character*255 b1,b2
      integer info,l,irc
      integer xeq

c     --------------------------------------
c     check if master and slave are the same
c     --------------------------------------
      call clean(b2)
      if (mynode.eq.pnode) then

c       ------------
c       copy locally
c       ------------
        b2='cp '
        l=3

      else

c       ----------------
c       copy from master
c       ----------------
        b2='scp '//masterid(1:li)//':'
        call llen(b2,l)

      end if

c     --------------------
c     write command string
c     --------------------
      b1=b2(1:l)//mydir(1:lm)//'/'//name//' '//tempdir(1:lt)
CCH      b1=b2(1:l)//workdir(1:lworkdir)//'/'//name//' '//tempdir(1:lt)
      call llen(b1,l)

c     ---------------
c     execute command
c     ---------------
      irc=xeq(b1(1:l))

c     ----------------------------------------------------------
c     error handling: store status value as last digit in <info>
c     ----------------------------------------------------------
      if (irc.GT.0) irc=2
      if (irc.LT.0) irc=3
      if (irc.EQ.0) irc=1
      info=10*info+irc

      return
      end

C****************************************************************************
C//SKIPC~

      subroutine getfile_pvmpi(name,tid,info)
c     ------------------------------------------------
c     get file through parallel interface (PVM or MPI)
c     renamed by J. Neugebauer, ETHZ, 01/2007
c     (former name: gfile)
c     ------------------------------------------------

      include 'defs.h'
      include 'param.h'

      character*(*) name
      integer tid,info
      character*255 b2
      character*40 name_sent
      character*255 card
      integer l,irc,lu,ibuf

      CALL clean(b2)
      b2=tempdir(1:lt)//'/'//name
      lu=1
      OPEN (UNIT=lu,FILE=b2,FORM='FORMATTED',STATUS='UNKNOWN',
     * IOSTAT=irc)
      IF (irc.NE.0) GOTO 999
      WRITE (iosl,*) '  getfile_pvmpi | receiving ',name
      CALL pgetstr('getfile_pvmpi: filename',name_sent,tid,40,440,irc)
      IF (irc.NE.0) GOTO 999
      WRITE (iosl,*) '       | ',name_sent
      IF (name_sent(1:6).NE.'%FILE '.OR. name_sent(7:40).NE.name) THEN
       WRITE (iosl,*) '       | wrong filename, expecting file <',
     *  name,'>'
       GOTO 999
      ENDIF
1     CONTINUE
      CALL clean(card)
      CALL pgetstr('getfile_pvmpi: card',card,tid,255,441,irc)
C@    WRITE (iosl,*) '       | {',irc,'} ',card(1:30)
C@    CALL flush(iosl)
      IF (irc.NE.0) GOTO 999
      IF (card(1:6).EQ.'%EOF% ') GOTO 999
      WRITE (lu,'(A)') card
      GOTO 1
999   CONTINUE
      IF (irc.GT.0) irc=2
      IF (irc.LT.0) irc=3
      IF (irc.EQ.0) irc=1
      info=10*info+irc
      CLOSE (UNIT=lu)
      RETURN
      END
C//END
