CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine islave

c     ==================================================================
c     purpose: initialize slave (child) processes
c     
c     history:
c       - re-written by J. Neugebauer, Uni Erlangen, 2003
c         (based on earlier numfreq version)
c       - modified by C. Herrmann, Uni Bonn, 02/2005 (intonly tool)
c       - modified by C. Herrmann, Uni Jena 04/05 (ADF interface)
c       - modified by C. Herrmann, Uni Jena 05/05 (Gaussian interface)
c       - modified by S. Luber, ETH Zurich 06/06 (Molpro interface)
c       - restructured by J. Neugebauer, ETH Zurich, 01/2007
c     ==================================================================

      INCLUDE 'defs.h'
      INCLUDE 'objects.h'
      INCLUDE 'comm.h'
      INCLUDE 'param.h'  
      INTEGER l,ibuf,info,mbuf,ierr,irc,xeq,w,ns,lf
      INTEGER intgr,iclean,icleanl,idalmw,nfex,mcharge,ir1,lsco
C#    INTEGER st(MPI_STATUS_SIZE)
C#    CHARACTER*(MPI_MAX_PROCESSOR_NAME) mpiname
      INTEGER*4 iready,iwhat,kvers
      logical snffast,udalinp
      CHARACTER*255 b1,b2,logfile,fname
      character*80 dalmw
      CHARACTER*40 mynode,pnode
      external nfex
      real*8 normalmode(maxmode,maxatoms*3),freq(maxmode)
      integer intonly,nummodes,i,j
      INCLUDE 'VERSION.h'  

c     --------------------------
c     initialize slave processes
c     --------------------------
      ns=mxwrk
      isuhf=0
      ridft=0

c     ------------------------------------------------------------------------
c     receive tempdir, jobdir, parent node name, current node name from master
c     ------------------------------------------------------------------------

      call initdir(logfile)

C#      call MPI_GET_PROCESSOR_NAME(mpiname,l,info)
C#      call mpierr('MPI_GET_PROCESSOR_NAME','islave',info)
C#      call clean(mynode)
C#      mynode=mpiname(1:l)

      call pgetstr('islave: pnode',pnode,tid0,40,407,info)

C#      call pputstr('islave: mynode',mynode,tid0,40,408,info)
C//SKIPC#1
      call pgetstr('islave: mynode',mynode,tid0,40,408,info)

      write (iosl,*) 'MYNODE=',mynode
      write (iosl,*) 'PNODE =',pnode

c     ------------------------------------------------------------------------
c     initialize memory manager
c     ------------------------------------------------------------------------
      call memmgr(1024 ,'memdbg',0,0,iosl)
      call memmgr(memini,'init',is,1,ns)

c     ------------------------------
c     copy and read snf_control file
c     ------------------------------
      info = 0
      write (iosl,*) 'islave: waiting for <snf_control>'
      call flush(iosl)
      call getfile('snf_control',mynode,pnode,tid0,info)
      write (iosl,*) 'return form getfile, info=',info
      call slen(tempdir(1:lt)//'/snf_control',lsco)
      write(iosl,*) 'islave:  reading <snf_control> from'
      write(iosl,*) tempdir(1:lt)//'/snf_control'
      call rscontrol(tempdir(1:lt)//'/snf_control',progno,calcno,ipol,
     *  omeg,snffast,iclean,icleanl,naus,mxomeg,cstep,lastqmatom,
     *  irescp,restadf,para,lsco)
      write(iosl,*) 'islave: PROGNO = ', progno
      write(iosl,*) 'islave: CALCNO = ', calcno
      call flush(iosl)

c     ---------------------------------
c     copy program-specific input files
c     ---------------------------------
      call copyfiles(info,mynode,pnode)

c     ----------------------------------------------------------------
c     input parameters determined -> copy program-specific input files
c     okok contains a "1" for every file to be copied, e.g.,
c     4 files = "1111"
c     ----------------------------------------------------------------

      call rscontrol2(normalmode,nummodes,intonly,freq,
     * tempdir(1:lt)//'/snf_control',iosl) 

      okok=nfex(progno,calcno,isuhf)
      write(iosl,*) 'okok: ',okok
      call flush(iosl)
      write (iosl,*) '>>> input files copied, results is ',info
      call flush(iosl)
      if(okok.le.0) call errex('islave | invalid okok value')

      iready=info
      write (b1,'(I5)') ivers
C@    call mnotify(9,mytid,iready,b1,5)

*---------------
* node ready ???  
*---------------
C#      w=MPI_COMM_WORLD
C:      call MPI_SSEND(iready,1,MPI_INTEGER,tid0,407,w,info)
C//SKIPC:1
C#      call MPI_SEND(iready,1,MPI_INTEGER,tid0,407,w,info)
C#      call mpierr('MPI_(S)SEND iready','islave',info)
C:      call MPI_SSEND(ivers,1,MPI_INTEGER,tid0,408,w,info)
C//SKIPC:1
C#      call MPI_SEND(ivers,1,MPI_INTEGER,tid0,408,w,info)
C#      call mpierr('MPI_(S)SEND ivers','islave',info)
C#      if (iready.NE.okok) THEN
C#        write (iosl,*) '>>> calling termcomm: iready=',iready
C#        call flush(iosl)
C#        call termcomm('islave: not ready')
C#      endif

C#52    call MPI_RECV(iwhat,1,MPI_INTEGER,tid0,409,w,st,info)
C#      call mpierr('MPI_RECV iwhat','islave',info)
C#      if (iwhat.EQ.1) CALL cslave(normalmode,freq,intonly,nummodes)
C#      if (iwhat.GE.0) GOTO 52

C//SKIPC#
        call PVMFINITSEND(PVMDATADEFAULT,MBUF)
        call PVMFPACK(INTEGER4, iready, 1, 1, INFO )
        if (info.ne.0) call pvmfperror('pack iready',info)
        kvers=ivers
        call PVMFPACK(INTEGER4, kvers, 1, 1, INFO )
        if (info.ne.0) call pvmfperror('pack ivers',info)
        call PVMFSEND( tid0, 0, INFO )
        if (info.ne.0) call pvmfperror('send iready',info)
        if (iready.ne.okok) then
          write (iosl,*) '>>> calling termcomm: iready=',iready
          call flush(iosl)
          call termcomm('islave: not ready')
        end if

*---------------------------------------------------------
* still missing: remove old scratch and restart files
*                of single point calculations
*---------------------------------------------------------
*----------------
* receive command
*----------------
42      continue 
        call PVMFRECV(tid0,0,ibuf)
        call PVMFUNPACK(INTEGER4,iwhat,1,1,info)
        write (iosl,*) '>>> iwhat =',iwhat
        call flush(iosl)
        write (iosl,*) 'islave: ',tempdir(1:lt)
        if(iwhat.eq.1) call cslave(normalmode,freq,intonly,nummodes)
        write(iosl,*) ' back from cslave, iwhat = ',iwhat
        call flush(iosl)
CDBG        if(intonly.ne.1) then
        if(iwhat.gt.-1) goto 42
CDBG        end if
C//END
99    continue
      write (iosl,*) '>>> finishing'
      call flush(iosl)
C@    info=xeq('rm -rf '//tempdir(1:lt))
      write (iosl,*) 'slave tid ',mytid,' stopping.'
      close (unit=iosl)
      if(icleanl.eq.1) then
        CALL slen(logfile,lf)
        info=xeq('rm '//logfile(1:lf))
      end if
      if(iclean.eq.1) then
        info=xeq('rm -r '//tempdir(1:lt))
      end if
      return
      end
