CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************

      program snfdcsr

c     ===============================================================
c     serial data collector for the program snf, a program to 
c     calculate vibrational spectra by means of numerical derivatives 
c     of gradients and property tensors provided by external 
c     electronic structure program.
c
c     supported programs:
c       - Turbomole
c       - Dalton
c       - ADF
c       - Gaussian98/Gaussian03
c       - Molpro
c
c     supported types of spectra:
c       - Infrared spectra
c       - Raman spectra (based on static / dynamic polarizabilities)
c       - Vibrational Raman optical activity (VROA) spectra
c
c     serial version by J. Neugebauer, ETHZ, 01/2007
c     ===============================================================

      include 'defs.h'
      include 'objects.h'
      include 'param.h'  

      integer ierr,ns,ipe,t1,t2,ltbuf,j
      logical nodeav
      character*255 tbuf

      include 'VERSION.h'

c     -----------------
c     start timer
c     -----------------
      call secs(t1)

      inrec=0
      ierr=0
      ipol=0
      naus=2
      pmch=0
      isuhf=0
      pvmmch=0
      ns=mxwrk

c     -----------------
c     initialize master
c     -----------------
      call iniparam
      iosl=idbg
      call memmgr(1024 ,'memdbg',0,0,iosl)
      call memmgr(memini,'init',is,1,ns)

c     ------------------------------
c     initialize the master process
c     ------------------------------
      call imaster

c     ----------------------------------
c     read input
c     ----------------------------------
      call rdparm

c     -------------------------------
c     read basic input files
c     -------------------------------
      call basic_input

c     --------------------
c     run master scheduler
c     --------------------
      call master

c     ---------------------------------
c     stop timer and print elapsed time
c     ---------------------------------
      call secs(t2)
      call clean(tbuf)
      call ptime(t2-t1,tbuf)
      call llen(tbuf,ltbuf)
      write(jtape,*)  
      write(jtape,'(500a)')  ('*',j=1,ltbuf+3+23)
      write(jtape,'(3a)')'** time used in snfdc: ',tbuf(1:ltbuf),' **'
      write(jtape,'(500a)')  ('*',j=1,ltbuf+3+23)

      end
