CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************

      SUBROUTINE master

c     ================================================
c     purpose: run the master process
c
c     history:
c     - modified by J. Neugebauer, Uni Erlangen, 2001
c     - modified by M. Reiher, Uni Erlangen, June 2003
c     - modified by C. Herrmann, Uni Bonn, 02/2005
c     ================================================

      include 'defs.h'
      include 'param.h'  
      include 'objects.h'
      include 'sadmch.h'

      integer p3,ize,ipe,iprc,info,i
      integer pdipder,pddip,puc,peig,pamass,ph
      integer intonly,nummodes

      character*8 dash

      data dash /'--------'/
      
c     ---------------------------------------------------------------
c     allocate storage for dipole, polarizability and gradient arrays
c     ---------------------------------------------------------------
      call memmgr(memgdw,'dipol   ',s8,pdip,3*naus*nvar)
      call memmgr(memgdw,'polar   ',s8,ppol,8*naus*nvar*mxomeg)
      call memmgr(memgdw,'gtenlao ',s8,pgtenlao,9*naus*nvar*mxomeg)
      call memmgr(memgdw,'gtenao  ',s8,pgtenao,9*naus*nvar*mxomeg)
      call memmgr(memgdw,'aten    ',s8,paten,27*naus*nvar*mxomeg)
      call memmgr(memgdw,'grad    ',s8,pgra,3*natoms*naus*nvar)
      call memmgr(memget,'istepfl ',is,pstepfl,nstept)
      call memmgr(memgdw,'nvmach  ',s8,pvmach,nstept)
      
      do i=1,nstept
        read (dash,'(A8)') s8(pvmach+i-1)
        is(pstepfl+i-1)=0
      end do

c     -------------------------------------------------------------------
c     in case of MPI, we have to set up all slaves for communication.
c     otherwise, we get into trouble if some of the slaves are not called.
c     -------------------------------------------------------------------
C#    p3=pmch
C#20  if (p3.eq.0) goto 21      
C#    call spawn(p3,idbg)
C#    p3=is(p3+sadmch_next)
C#    goto 20
C#21  continue

c     -----------------
c     read restart file
c     -----------------
      call rrestart(s8(pvmach),s8(pdip),s8(ppol),s8(pgtenlao),
     &              s8(pgtenao),s8(paten),s8(pgra),is(pstepfl),i,
     &              intonly,nummodes)
      write (idbg,*) ' master| <restart> read'

      if (i.ne.nstept) then
        call memmgr(memgdw,'g       ',s8,p3,nvar)
C//SKIPC~
        call parstuff(s8(p3),s8(pdip),s8(ppol),s8(pgtenlao),s8(pgtenao),
     &       s8(paten),s8(pgra),is(pstepfl),s8(pvmach),intonly,nummodes)
C//END
C~      call serstuff(s8(p3),s8(pdip),s8(ppol),s8(pgtenlao),s8(pgtenao),
C~   &       s8(paten),s8(pgra),is(pstepfl),s8(pvmach),intonly,nummodes)
        call memmgr(memfdw,'g       ',s8,p3,nvar)
      end if

c     ----------------------------------
c     check if all points are calculated
c     ----------------------------------
      call howmany(ize,ipe,iprc,is(pstepfl))
      if(ize.gt.0.or.ipe.gt.0) then
        write(*,*) 
        write(*,*) 'CAUTION: NOT ALL CALCULATIONS DONE'
        write(*,*) '(maybe there are no nodes present'
        write(*,*) 'in the parallel virtual machine)'
        write(*,*) 
      end if

c     ------------------
c     deallocate storage
c     ------------------
      call memmgr(memfdw,'dipol   ',s8,pdip,3*naus*nvar)
      call memmgr(memfdw,'polar   ',s8,ppol,8*naus*nvar*mxomeg)
      call memmgr(memfdw,'gtenlao ',s8,pgtenlao,9*naus*nvar*mxomeg)
      call memmgr(memfdw,'gtenao  ',s8,pgtenao,9*naus*nvar*mxomeg)
      call memmgr(memfdw,'aten    ',s8,paten,27*naus*nvar*mxomeg)
      call memmgr(memfdw,'grad    ',s8,pgra,3*natoms*naus*nvar)
      call memmgr(memfre,'istepfl ',is,pstepfl,nstept)
      call flush(idbg)
      
      return
      end 
