CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine parstuff(g,dip,pol,gtenlao,gtenao,aten,gra,stepfl,
     *                    vmach,intonly,nummodes)

c     =======================================================================
c     purpose: control the parallel execution of the displaced single points,
c              send input data for new single points and gather results from
c              finished calculations.
c
c     history:
c     - modified by M. Reiher, Uni Erlangen, June 2003
c     - modified by C. Herrmann, Uni Bonn, 02/2005
c     - modified by J. Neugebauer, ETH Zurich, 01/2007
c     =======================================================================

      include 'defs.h'
      include 'comm.h'
      include 'param.h'  
      include 'objects.h'
      include 'sadmch.h'

c     ---------------------------------------------------------------------------
c     variables:
c
c       g           R8         short gradient buffer (1:nvar)
c       dip         R8         dipole buffer
c       pol         R8         polarizability buffer
c       gtenlao     R8         G/LAO tensor buffer for VROA
c       gtenao      R8         G/AO tensor buffer for VROA
c       aten        R8         A tensor buffer for VROA
c       gra         R8         gradient buffer
c       stepfl      I4         status flags for each point
c       vmach       R8         machine name for each point
c       intonly     integer    flag for intensities-only mode (on = 1; off = 0)
c       nummodes    integer    number of normal modes in intensities-only mode
c     ---------------------------------------------------------------------------

      integer intonly, nummodes
      integer stepfl(*)
      real*8 dipintonly(nummodes,naus)
      real*8 g(nvar),dip(*),pol(*),gra(*),vmach(*)
      real*8 gtenlao(*),gtenao(*),aten(*)

c     -----------------------------------------------------------------------------
c     declare integers to be packed and unpacked as INTEGER*4 for T3E compatibility
c     -----------------------------------------------------------------------------
      integer i,k,l,idum,mbuf,ize,ipe,iprc,p,nnode,info

C#    integer st(MPI_STATUS_SIZE)
C#    integer w
C#    DATA w /MPI_COMM_WORLD/

888   p=pmch
20    if (p.eq.0) goto 21      
      call howmany(ize,ipe,iprc,stepfl)
C@    write (idbg,*) '@parstuff ize, ipe, iprc ',ize,ipe,iprc,p
C//SKIPC#1
      if (ize.gt.0) call spawn(p,jtape)
C#    if (ize.eq.0 .and. is(p+sadmch_nodf).EQ.sadmch_rdy) then
C#      call remnode(p,'shutdown[idle]',0,stepfl)
C#    end if
      call fsys(stepfl)
      call grant_ctok
      call readcmd
      call trace_stat
      call ssend(p,dip,pol,gtenlao,gtenao,aten,gra,stepfl,vmach,
     *           intonly,nummodes,dipintonly)
      call getres(p,g,dip,pol,gtenlao,gtenao,aten,gra,stepfl,vmach,
     *            intonly,nummodes,dipintonly)
      p=is(p+sadmch_next)
      goto 20

21    continue
      call sleep(1)

      call howmany(ize,ipe,iprc,stepfl)
C@    write (idbg,*) '@parstuff ize, ipe, iprc ',ize,ipe,iprc
      call flush(idbg)

      if (ize+ipe.ne.0 .and. iprc.ne.0) goto 888

      return
      end
