CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************

      subroutine rdparm

c     ============================================================
c     read input parameters; called from imaster from file DCINPUT
c     
c     adapted for serial version by J. Neugebauer, ETHZ, 01/2007
c     ============================================================

      include 'defs.h'
      include 'param.h'  
      include 'objects.h'

      integer i,intgr,ierr,l,lmy
      character*255 b1
      external intgr
      
c     ----------------------------------
c     read input, unless already done so
c     ----------------------------------
      IF (inrec.EQ.0) call readin

      write (*,*) 
      write (*,*) '========================================='
      write (*,*) '== SNFDC INPUT PARAMETERS FROM DCINPUT =='
      write (*,*) '========================================='
      write (*,*) 
      ierr=0

c     ------------------------------------------------------
c     MYDIR: the current working directory
c     ------------------------------------------------------
      mydir='*MYDIR?'
      call rdir('MYDIR',mydir,lm,0,ierr)
      write (*,*) 'MYDIR        ',mydir(1:lm)

c     ------------------------------------------------------
c     TITLE: title of calculation
c     ------------------------------------------------------
      call readv('TITLE',title,mydir(1:lm))
      write (*,*) 'TITLE        ',title(1:60)

c     ------------------------------------------------------
c     TMPDIR: directory where calculations are performed
c     ------------------------------------------------------
      tempdir=mydir
      tempdir(lm+1:lm+4)='/tmp'
      call rdir('TMPDIR',tempdir,lt,0,ierr)
      write (*,*) 'TMPDIR       ',tempdir(1:lt)

c     ------------------------------------------------------
c     LOGDIR: directory where logfiles are written
c     ------------------------------------------------------
      logdir=mydir
      logdir(lm+1:lm+4)='/log'
      call rdir('LOGDIR',logdir,ll,1,ierr)
      write (*,*) 'LOGDIR       ',logdir(1:ll)

c     ------------------------------------------------------
c     ARPATH: path for archiving (fine-grained restarts)
c     ------------------------------------------------------
      call readv('ARPATH',arpath,' ')
      call llen(arpath,lar)
      IF (lar.GT.0) write (*,*) 'ARPATH       ',arpath(1:lar)

c     ------------------------------------------------------
c     PROGDIR: path to electronic structure program binaries
c     ------------------------------------------------------
      call readv('PROGDIR',progdir,' ')
      call llen(progdir,lb)
      IF (lb.GT.0) write (*,*) 'PROGDIR      ',progdir(1:lb)
      IF (lb.GT.0) THEN
       progdir(lb+1:lb+1)='/'
       lb=lb+1
      ENDIF

C//SKIPC~
c     ------------------------------------------------------
c     DCPATH: full name of snfdc binary to be used on slaves
c     ------------------------------------------------------
      call readv('DCPATH',dcdir,'snfdc')
      call llen(dcdir,ldc)
      write (*,*) 'DCPATH       ',dcdir(1:ldc)

c     ------------------------------------------------------
c     CHOOSENOPT: options for choose_nodes script
c     ------------------------------------------------------
      call clean(cnopt)
      call rkeystr('DCINPUT','CHOOSENOPT',cnopt)
      call llen(cnopt,lnopt)
      write (*,*) 'CHOOSENOPT   ',cnopt(1:lnopt)
C//END

c     ------------------------------------------------------
c     CRAP_OK: override safety checks for certain parameters
c              (e.g., scfconv, cstep)
c     ------------------------------------------------------
      icrap=0
      call readv('CRAP_OK',b1,'no')
      call llen(b1,l)
      write (*,*) 'CRAP_OK      ',b1(1:l)
      IF (b1(1:6).NE.'no') icrap=1

c     --------------------------------------------------------
c     UPDATE: options for how to update the parallel machinery
c     --------------------------------------------------------
      call readv('UPDATE',b1,'3')
      call llen(b1,l)
      updopt=intgr(b1,l,20)
C//SKIPC~16
C//SKIPC#14
      if(updopt.ne.3) then
        write (*,*) 'UPDATE       ',b1(1:l)
        write (*,*) 'virtual machine update option(s) [UPDATE] :'
        if (MOD(updopt,2).GE.1) then
          write (jtape,*) 'remove        processes on vbsy nodes'
        else
          write (jtape,*) 'do not remove processes on vbsy nodes'
        end if
        if (MOD(updopt,4).GE.2) then
          write (jtape,*) 'add           processes on idle nodes'
        else
          write (jtape,*) 'do not add    processes on idle nodes'
        end if
      end if
C#
C~

c     --------------------------------------------------------
c     PREFIX: prefix for logfiles
c     --------------------------------------------------------
      call readv('PREFIX',b1,'TMP%')
      call llen(b1,l)
      write (*,*) 'PREFIX       ',b1(1:l)
      IF (l.GT.0) THEN
       logdir=logdir(1:ll)//b1(1:l)
       ll=ll+l
      ENDIF
      IF (ierr.NE.0) call errex('error encountered in input')

c     -------------------------------------
c     MXSTEP: maximum no. of steps per node
c             (this is a debugging option)
c     -------------------------------------
C@    call readv('MXSTEP',b1,'0')
C@    i=0
C@    mxstep=intgr(b1,i,20)
C@    IF (mxstep.GT.0) THEN
C@     write (jtape,*) 'maximal number of steps per node :',mxstep
C@    ENDIF

c     --------------------------------------------------------
c     FSYS: 0     - system reporting off (default)
c           other - system reporting on (default)
c     --------------------------------------------------------
      call readv('FSYS',b1,'0')
      call llen(b1,l)
      write (*,*) 'FSYS         ',b1(1:l)
      i=0
      iprintf=intgr(b1,i,20)

c     --------------------------------------------------
c     CHANNELS: maximum number of communication channels
c               for archiving (fine-grained restarts)
c     --------------------------------------------------
C//SKIPC~
      call readv('CHANNELS',b1,'0')
      call llen(b1,l)
      write (*,*) 'CHANNELS     ',b1(1:l)
      i=0
      ierr=intgr(b1,i,20)
      IF (ierr.GT.0) call setmax_ctok(ierr)
C//END

      call clean(restart)
      restart=mydir(1:lm)//'/restart'
      dcstat =mydir(1:lm)//'/TMPdcstat'

      write (*,*)
      write (*,*) '========================================='

      return
      end
