CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine serres(nit,cpu,wall,gcpu,gwall,e,d,g,po,gtenla,gtena,
     &  ate,ia,dip,gra,pol,gtenlao,gtenao,aten,stepfl,vmach,nummodes,
     &  intonly)

c     =================================================================
c     write results from step <ia> to restart file in case of 
c     serial run.
c 
c     written by J. Neugebauer, ETHZ, 01/2007
c     based on routines getres and serres from akira
c
c     CAUTION: always synchronize with getres
c     =================================================================

      include 'defs.h'
      include 'param.h'  

      integer stepfl(*)
      integer ibuf,m,info,icmp,iaus,now,tid,ia,i,j,jomeg
      integer*4 nit
      character*24 date
      character*40 msg,name
      real*8 g(nvar)
      real*8 dip(3,naus,nvar)
      real*8 pol(mxomeg,8,naus,nvar)
      real*8 gra(nvar,naus,nvar)
      real*8 e,d(3),po(8,mxomeg)

      real*8 gtenlao(mxomeg,9,naus,nvar),gtenao(mxomeg,9,naus,nvar),
     *       aten(mxomeg,27,naus,nvar)
      real*8 gtenla(9,mxomeg),gtena(9,mxomeg),ate(27,mxomeg)
      logical vroaflag
      integer ipoldum
      integer intonly,nummodes

      real*8 cpu,wall,gcpu,gwall
      real*8 gsum,vmach(*)

      vroaflag=.false.
      ipoldum=ipol
      if (ipol-1000.ge.0) then
        vroaflag=.true.
        ipoldum=ipol-1000
        if (ipoldum.eq.0) ipoldum=1
      end if

C@    write (jtape,'(4D18.8)') e
C@    write (jtape,*) ' '
C@    write (jtape,'(4D18.8)') (d(j),j=1,3)
C@    write (jtape,*) ' '
C@    write (jtape,'(4D18.8)') (g(j),j=1,nvar)
C@    write (jtape,*) ' '
C@    write (jtape,'(4D18.8)') (po(j),j=1,8*ipoldum)
C@    write (jtape,*) ' '
C@    write (jtape,'(4D18.8)') (gtenla(j),j=1,8*ipoldum)
C@    write (jtape,*) ' '
C@    write (jtape,'(4D18.8)') (gtena(j),j=1,8*ipoldum)
C@    write (jtape,*) ' '
C@    write (jtape,'(4D18.8)') (ate(j),j=1,8*ipoldum)
C@    write (jtape,*) ' '
            
      gsum=0.0d0
      do j=1,nvar
         gsum=gsum+g(j)**2
      end do
      write(*,'(A)')       '----------------------------------------'
      write(*,'(A,i15)')   'STEP NO.            : ',ia
      write(*,'(A,f15.2)') 'CPU TIME            : ',cpu+gcpu
      write(*,'(A,f15.2)') 'WALL TIME           : ',wall+gwall
      write(*,'(A,i15)')   'NO. OF ITERATIONS   : ',nit
      write(*,'(A,f15.8)') 'ELEC. ENERGY        : ',e
      write(*,'(A)')       '----------------------------------------'
       
c     --------------------------------------------------------------
c     the energies are not used beyond this point, and in particular
c     not written to the restart file
c     --------------------------------------------------------------

c     -------------------------------------------------------------------
c     CH: removed energy check for ADF calculations (ADF does not provide
c     total energies)
c     -------------------------------------------------------------------

      if(((progno.ne.4.and.e.ge.0.0d0).or.gsum.lt.1.d-16.or.gsum.gt.1.d0
     *  ).and.nummodes.eq.0) then
        if (e.ge.0.D0) then
          write (jtape,'(A20,D20.10)') ' invalid energy:',e
          call errex('invalid energy')
        else
          write (jtape,'(A20,D20.10)') ' invalid gsum:',gsum
          call errex('invalid gsum')
        end if
c       ---------------------
c       shut down the process
c       ---------------------
        call errex('wrong results')
        return
      else
        call fdate(date)
        write (jtape,109) date,' (serial mode) <- ',masterid(1:9),ia
109     format(a,a,' ',a,' return step ',i3)
      end if

      stepfl(ia)=1
      iaus=mod(ia,naus)
      if (iaus.eq.0) iaus=naus
      m=ia/naus
      if(mod(ia,naus).ne.0) m=m+1
      do icmp=1,3
       dip(icmp,iaus,m)=d(icmp)
      end do
      if (.not.vroaflag) then
        do jomeg=1,ipol
          do icmp=1,8
            pol(jomeg,icmp,iaus,m)=po(icmp,jomeg)
          enddo
        enddo
      else 
        do jomeg=1,ipoldum
          do icmp=1,8
            pol(jomeg,icmp,iaus,m)=po(icmp,jomeg)
          enddo
          do icmp=1,9
            gtenlao(jomeg,icmp,iaus,m)=gtenla(icmp,jomeg)
            gtenao(jomeg,icmp,iaus,m)=gtena(icmp,jomeg)
          enddo
          do icmp=1,27
            aten(jomeg,icmp,iaus,m)=ate(icmp,jomeg)
          enddo
        enddo
      end if
      do icmp=1,nvar
        gra(icmp,iaus,m)=g(icmp)
      end do

      call wrestart(vmach,dip,pol,gtenlao,gtenao,aten,gra,stepfl,
     *              intonly,nummodes)

      return
      end
