CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************

      subroutine serstuff(g,dip,pol,gtenlao,gtenao,aten,gra,stepfl,
     *                    vmach,intonly,nummodes)

c     =======================================================================
c     purpose: control the serial execution of the displaced single points,
c              send input data for new single points and gather results from
c              finished calculations.
c
c     history:
c     - written by J. Neugebauer, ETH Zurich, 01/2007, based on parstuff
c       and the akira version of serstuff
c     =======================================================================

      include 'defs.h'
      include 'param.h'  
      include 'objects.h'

c     ---------------------------------------------------------------------------
c     variables:
c
c       g           R8         short gradient buffer (1:nvar)
c       dip         R8         dipole buffer
c       pol         R8         polarizability buffer
c       gtenlao     R8         G/LAO tensor buffer for VROA
c       gtenao      R8         G/AO tensor buffer for VROA
c       aten        R8         A tensor buffer for VROA
c       gra         R8         gradient buffer
c       stepfl      I4         status flags for each point
c       vmach       R8         machine name for each point
c       intonly     integer    flag for intensities-only mode (on = 1; off = 0)
c       nummodes    integer    number of normal modes in intensities-only mode
c     ---------------------------------------------------------------------------

      integer intonly,nummodes,info,lsco,i,ize,ipe,iprc
      integer iclean,icleanl,lf,xeq
      external xeq
      integer stepfl(*)
      real*8 g(nvar),dip(*),pol(*),gra(*),vmach(*)
      real*8 gtenlao(*),gtenao(*),aten(*)
      real*8 normalmode(maxmode,maxatoms*3),freq(maxmode)
      character*8 localhost
      character*255 b1,logfile
      character*40 mynode,pnode
      character*24 date
      logical first,snffast
      save first

      data first / .true. /

c     ------------------------------------------------------------------------
c     serstuff carries out the tasks performed by the pair of routines
c     parstuff-islave in parallel runs. That includes creating directories
c     and copying input files.
c
c     NOTE: always synchronize with islave.r, spawn.r, basic_input.r (okok !!)
c     ------------------------------------------------------------------------

      call clean(mynode)
      call clean(pnode)
      mynode='localhost'
      pnode='localhost'
      localhost='loclhost'

      if(first) then
        first = .false.
        call initdir(logfile)

c       ------------------------------
c       copy and read snf_control file
c       ------------------------------
        info = 0
        write (idbg,*) 'serstuff: waiting for <snf_control>'
        call flush(idbg)
        call getfile('snf_control',mynode,pnode,tid0,info)
        write (idbg,*) 'return form getfile, info=',info

        call slen(tempdir(1:lt)//'/snf_control',lsco)
        write(idbg,*) 'serstuff:  reading <snf_control> from'
        write(idbg,*) tempdir(1:lt)//'/snf_control'
        call rscontrol(tempdir(1:lt)//'/snf_control',progno,calcno,ipol,
     *   omeg,snffast,iclean,icleanl,naus,mxomeg,cstep,lastqmatom,
     *   irescp,restadf,para,lsco)
        write(idbg,*) 'serstuff: PROGNO = ', progno
        write(idbg,*) 'serstuff: CALCNO = ', calcno
        call flush(idbg)

c       ---------------------------------
c       copy program-specific input files
c       ---------------------------------
        call copyfiles(info,mynode,pnode)
       
c       ----------------------------------------------------------------
c       input parameters determined -> copy program-specific input files
c       okok contains a "1" for every file to be copied, e.g.,
c       4 files = "1111"
c       note: okok is known from routine basic_input in serial run
c       ----------------------------------------------------------------
       
        call rscontrol2(normalmode,nummodes,intonly,freq,
     *   tempdir(1:lt)//'/snf_control',idbg)

      end if
      write(idbg,*) 'info, okok: ',info,okok
      if(info.ne.okok) call errex('** serstuff: info .ne. okok')

      do
        call howmany(ize,ipe,iprc,stepfl)
        if(ize.eq.0 .and. ipe.eq.0) exit

c       -------------------------------------------------
c       determine current step
c       CAUTION: at the beginning of the calculation, all
c       unfinished steps must be set from -1 to 0 !!!
c       -------------------------------------------------

        do i=1,nstept
          if(stepfl(i).eq.0) then
            isstep=i
            stepfl(i)=-1
            read(localhost(1:8),'(A8)') vmach(i)
            exit
          end if
          if(i.eq.nstept) exit
        end do

        write(idbg,*) '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'
        write(idbg,'(1x,a,i5,a,a8)') '>>> snfdc: starting step ',
     &    isstep,' on ',vmach(isstep)
        write(idbg,*) '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'
        call fdate(date)
        write(*,'(A,A,1X,A,A,I4,A,I4)')
     &    date,' (serial mode) -> ',mynode(1:9),' sending step',isstep

        call cslave(normalmode,freq,intonly,nummodes)

      end do

      if(icleanl.eq.1) then
        CALL slen(logfile,lf)
        info=xeq('rm '//logfile(1:lf))
      end if
      if(iclean.eq.1) then
        info=xeq('rm -r '//tempdir(1:lt))
      end if

      return
      end
